local HC = require 'HardonCollider'
local loader = require "AdvTiledLoader/Loader"
loader.path = "maps/"

local allSolidTiles

function love.load()
    -- initialize library
    Collider = HC(100, on_collision, collision_stop)
    bang = string.format("lol")
    --particle
    flashimg = love.graphics.newImage("particle/flash.png")
    flash = love.graphics.newParticleSystem( flashimg, 50 )
    emitterpos = Collider:addPoint(800/2+19,600/2+4)
    flash:setEmissionRate          (50)
    flash:setLifetime              (1)
    flash:setParticleLife          (0.2)
    flash:setPosition              (50, 50)
    flash:setDirection             (0)
    flash:setSpread                (1)
    flash:setSpeed                 (50, 100)
    flash:setGravity               (0)
    flash:setRadialAcceleration    (10)
    flash:setTangentialAcceleration(0)
    flash:setSize                  (1)
    flash:setSizeVariation         (0.5)
    flash:setRotation              (0)
    flash:setSpin                  (0)
    flash:setSpinVariation         (0)
    flash:stop()

    --add the dude
    dude = love.graphics.newImage("char/dude.png")
    smp = love.graphics.newImage("char/smp.png")
    selImg = love.graphics.newImage("selection/sel.png")
    speed = 150
    circ = Collider:addCircle(800/2, 600/2, 13)

    --screensettings
    gw, gh = love.graphics.getWidth(), love.graphics.getHeight()
    --create bullets table
    bullets = {}

    --load the level
    map = loader.load("level.tmx")
    allSolidTiles = findSolidTiles(map)
end

function love.update(dt)
    --bullet stuff
    for i = #bullets, 1, -1 do -- reverse-order trick
        local b = bullets[i]
        local bx, by = b:center()
        -- Destroy off-screen instances
--        if b:collidesWith(rect) then 
--            table.remove(bullets, i)
--            Collider:remove(b)
--            Collider:remove(b.prev)
--            break
--        end
        if bx < 0 or bx > gw or by < 0 or by > gh then
            table.remove(bullets, i)
            Collider:remove(b)
            Collider:remove(b.prev)
            --Collider:removeFromGroup('bullets', b, b.prev)
        -- Move the bullet
        else
            b.prev:moveTo(x, y)
            b:move(b.xvel * dt, b.yvel * dt)
            bx2, by2 = b:center()
        end
    end


    if love.keyboard.isDown("d") then
        circ:move(speed*dt,0)
        emitterpos:move(speed*dt,0)
    elseif love.keyboard.isDown("a") then
        circ:move(-speed*dt,0)
        emitterpos:move(-speed*dt,0)
    end

    if love.keyboard.isDown("s") then
        circ:move(0,speed*dt)
        emitterpos:move(0,speed*dt)
    elseif love.keyboard.isDown("w") then
        circ:move(0,-speed*dt)
        emitterpos:move(0,-speed*dt)
    end

--    if not love.keyboard.isDown("down") and not love.keyboard.isDown("left") and not love.keyboard.isDown("right") and not love.keyboard.isDown("up") then
--        speed = 100
--    end

    lol = dt
    x, y = love.mouse.getPosition( )

    xc, yc = circ:center()

    circAngle = getAngle(x,y ,xc,yc)

    circ:setRotation(circAngle, xc, yc)

    -- check for collisions
    Collider:update(dt)


    -- particle flash
    emitterpos:setRotation(circAngle-math.pi, xc, yc)
    xe, ye = emitterpos:center()
    flash:setPosition(xe, ye)
    flash:setDirection(circAngle-math.pi)
    if love.mouse.isDown("l") then
        shot = shot + 1
        if shot%5 == 4 or shot == 1 then
            flash:start()
            shoot(x, y)
        end
    elseif not love.mouse.isDown("l") then
        flash:stop()
        shot = 0
    end
    flash:update(dt)
end

function love.draw()
    --draw map
    map:draw()
    -- print messages
    love.graphics.print(getDegree(circAngle),0,0)
    love.graphics.print(xe,0,15)
    love.graphics.print(ye,0,30)
    love.graphics.print(love.timer.getFPS(),0,45)
    --love.graphics.print(shot%5,0,60)
    --love.graphics.print(bang,0,75)
    -- shapes can be drawn to the screen
    love.graphics.setColor(255,255,255)
    love.graphics.draw(flash, 0, 0)
   -- circ:draw('line')
    emitterpos:draw('line')
    love.graphics.draw(dude, xc, yc, circAngle-math.pi, 1, 1, 12, 16)
    love.graphics.draw(smp, xc, yc, circAngle-math.pi, 1, 1, 13, 16)

    for i, b in ipairs(bullets) do
        -- Current position
        love.graphics.setColor(0xff, 0xff, 0xff)
        b:draw('line')
    end

    --
    if col then 
        love.graphics.draw(selImg, cx, cy, _, _, _, 8, 8)
    end 

end

-- this is called when two shapes stop colliding
function collision_stop(dt, shape_a, shape_b)
    col = false
end

function on_collision(dt, shape_a, shape_b, mtv_x, mtv_y)
    -- sort out which one our hero shape is
    local hero_shape, tileshape
    if shape_a == circ and shape_b.type == "tile" then
        hero_shape = shape_a
        colshape = shape_b
    elseif shape_b == dude and shape_a.type == "tile" then
        hero_shape = shape_b
        colshape = shape_a
    else
        -- none of the two shapes is a tile, return to upper function
        return
    end
    cx, cy = colshape:center()

    col = true
    hero_shape:move(mtv_x, 0)
    hero_shape:move(0, mtv_y)
end

function getAngle (x1, y1, x2, y2)
    dx = x2 - x1;
    dy = y2 - y1;
    return math.atan2(dy,dx);
end

function getDegree (r)
    return r * 180/math.pi
end

function findSolidTiles(map)
    
    local collidable_tiles = {}
    
    -- get the layer that the tiles are on by name
    local layer = map.tl["ground"]
    
    for tileX=1,map.width do
        for tileY=1,map.height do
            
            local tile
            
            if layer.tileData[tileY] then
                tile = map.tiles[layer.tileData[tileY][tileX]]
            end
            
            if tile and tile.properties.solid then
                local ctile = Collider:addRectangle((tileX-1)*16,(tileY-1)*16,16,16)
                ctile.type = "tile"
                Collider:addToGroup("tiles", ctile)
                Collider:setPassive(ctile)
                table.insert(collidable_tiles, ctile)
            end
            
        end
    end
    
    return collidable_tiles
end

function shoot(x, y)
    local speed = 750
    local ex, ey = emitterpos:center()
    local bullet = Collider:addPoint(ex, ey)
    bullet.name = 'bullet'
    local dir = math.atan2(y - ey, x - ex)
    bullet.dir = dir
    bullet.xvel = math.cos(dir) * speed
    bullet.yvel = math.sin(dir) * speed
    bullet.prev = Collider:addPoint(ex, ey)
    bullet.prev.name = 'bullet'
    Collider:addToGroup('bullets', bullet, bullet.prev)
    -- Add to instances table
    table.insert(bullets, bullet)
end